import * as _backstage_backend_plugin_api from '@backstage/backend-plugin-api';
import { LoggerService, DatabaseService, PermissionsService, HttpAuthService, DiscoveryService, SchedulerService, AuthService, UrlReaderService } from '@backstage/backend-plugin-api';
import { FactRetrieverRegistry as FactRetrieverRegistry$1, PersistenceContext as PersistenceContext$1, FactRetriever, FactLifecycle, FactRetrieverRegistration, FactChecker, FactCheckerFactory } from '@backstage-community/plugin-tech-insights-node';
import { HumanDuration } from '@backstage/types';
import { Duration } from 'luxon';
import express from 'express';
import { Config } from '@backstage/config';
import { Check, CheckResult } from '@backstage-community/plugin-tech-insights-common';

/**
 * The tech-insights backend plugin.
 *
 * @public
 */
declare const techInsightsPlugin: _backstage_backend_plugin_api.BackendFeature;

/**
 * @public
 * @deprecated Use FactRetrieverRegistry from `@backstage-community/plugin-tech-insights-node` instead.
 */
type FactRetrieverRegistry = FactRetrieverRegistry$1;
/**
 * @public
 * @deprecated Use PersistenceContext from `@backstage-community/plugin-tech-insights-node` instead.
 */
type PersistenceContext = PersistenceContext$1;

/**
 * @public
 *
 * @param cadence - cron expression to indicate when the fact retriever should be triggered
 * @param factRetriever - Implementation of fact retriever consisting of at least id, version, schema and handler
 * @param lifecycle - Optional lifecycle definition indicating the cleanup logic of facts when this retriever is run
 * @param timeout - Optional duration to determine how long the fact retriever should be allowed to run, defaults to 5 minutes
 *
 */
type FactRetrieverRegistrationOptions = {
    cadence: string;
    factRetriever: FactRetriever;
    lifecycle?: FactLifecycle;
    timeout?: Duration | HumanDuration;
    initialDelay?: Duration | HumanDuration;
};
/**
 * @public
 *
 * A helper function to construct fact retriever registrations.
 * @param cadence - Cron expression to indicate when the fact retriever should be triggered
 * @param factRetriever - Implementation of fact retriever consisting of at least id, version, schema and handler
 * @param lifecycle - Optional lifecycle definition indicating the cleanup logic of facts when this retriever is run
 * @param timeout - Optional duration to determine how long the fact retriever should be allowed to run, defaults to 5 minutes
 * @param initialDelay - Optional initial delay to determine how long the fact retriever should wait before the initial run, defaults to 5 seconds
 *
 *
 * @remarks
 *
 * Cron expressions help:
 * ┌────────────── second (optional)
 # │ ┌──────────── minute
 # │ │ ┌────────── hour
 # │ │ │ ┌──────── day of month
 # │ │ │ │ ┌────── month
 # │ │ │ │ │ ┌──── day of week
 # │ │ │ │ │ │
 # │ │ │ │ │ │
 # * * * * * *
 *
 * Valid lifecycle values:
 * \{ ttl: \{ weeks: 2 \} \} -- This fact retriever will remove items that are older than 2 weeks when it is run
 * \{ maxItems: 7 \} -- This fact retriever will leave 7 newest items in the database when it is run
 *
 */
declare function createFactRetrieverRegistration(options: FactRetrieverRegistrationOptions): FactRetrieverRegistration;

/**
 * @public
 *
 * FactRetrieverEngine responsible scheduling and running fact retrieval tasks.
 */
declare abstract class FactRetrieverEngine {
    /**
     * Schedules fact retriever run cycles based on configuration provided in the registration.
     *
     * Default implementation uses SchedulerService to handle scheduling. This function can be called multiple
     * times, where initial calls schedule the tasks and subsequent invocations update the schedules.
     */
    abstract schedule(): Promise<void>;
    /**
     * Schedules single fact retriever run cycles based on configuration provided in the registration.
     *
     * Default implementation defers to scheduling all jobs
     */
    scheduleJob(_: string): Promise<void>;
    /**
     * Provides possibility to manually run a fact retriever job and construct fact data
     *
     * @param ref - Reference to the task name stored in the executor database. By convention this is the fact retriever id
     */
    abstract triggerJob(ref: string): Promise<void>;
    /**
     * Exposes fact retriever job configuration information about previous and next runs and schedule
     *
     * @param ref - Reference to the task name stored in the executor database. By convention this is the fact retriever id
     */
    abstract getJobRegistration(ref: string): Promise<FactRetrieverRegistration>;
}

/**
 * Generates facts which indicate the completeness of entity metadata.
 *
 * @public
 */
declare const entityMetadataFactRetriever: FactRetriever;

/**
 * Generates facts which indicate the quality of data in the spec.owner field.
 *
 * @public
 */
declare const entityOwnershipFactRetriever: FactRetriever;

/**
 * Generates facts related to the completeness of techdocs configuration for entities.
 *
 * @public
 */
declare const techdocsFactRetriever: FactRetriever;

/**
 * A Container for persistence context initialization options
 *
 * @public
 */
type PersistenceContextOptions = {
    logger: LoggerService;
};
/**
 * A factory function to construct persistence context for running implementation.
 *
 * @public
 */
declare const initializePersistenceContext: (database: DatabaseService, options: PersistenceContextOptions) => Promise<PersistenceContext$1>;

/**
 * @public
 *
 * RouterOptions to construct TechInsights endpoints
 * @typeParam CheckType - Type of the check for the fact checker this builder returns
 * @typeParam CheckResultType - Type of the check result for the fact checker this builder returns
 */
interface RouterOptions<CheckType extends Check, CheckResultType extends CheckResult> {
    /**
     * Optional FactChecker implementation. If omitted, endpoints are not constructed
     */
    factChecker?: FactChecker<CheckType, CheckResultType>;
    /**
     * TechInsights PersistenceContext. Should contain an implementation of TechInsightsStore
     */
    persistenceContext: PersistenceContext$1;
    /**
     * Backstage config object
     */
    config: Config;
    /**
     * Implementation of Winston logger
     */
    logger: LoggerService;
    /**
     * Implementation of PermissionsService
     */
    permissions: PermissionsService;
    /**
     * Implementation of HttpAuthService
     */
    httpAuth: HttpAuthService;
}
/**
 * @public
 *
 * Constructs a tech-insights router.
 *
 * Exposes endpoints to handle facts
 * Exposes optional endpoints to handle checks if a FactChecker implementation is passed in
 *
 * @param options - RouterOptions object
 */
declare function createRouter<CheckType extends Check, CheckResultType extends CheckResult>(options: RouterOptions<CheckType, CheckResultType>): Promise<express.Router>;

/**
 * @public
 * @typeParam CheckType - Type of the check for the fact checker this builder returns
 * @typeParam CheckResultType - Type of the check result for the fact checker this builder returns
 *
 * Configuration options to initialize TechInsightsBuilder. Generic types params are needed if FactCheckerFactory
 * is included for FactChecker creation.
 */
interface TechInsightsOptions<CheckType extends Check, CheckResultType extends CheckResult> {
    /**
     * Optional collection of FactRetrieverRegistrations (required if no factRetrieverRegistry passed in).
     * Used to register FactRetrievers and their schemas and schedule an execution loop for them.
     *
     * Not needed if passing in your own FactRetrieverRegistry implementation. Required otherwise.
     */
    factRetrievers?: FactRetrieverRegistration[];
    /**
     * Optional factory exposing a `construct` method to initialize a FactChecker implementation
     */
    factCheckerFactory?: FactCheckerFactory<CheckType, CheckResultType>;
    /**
     * Optional FactRetrieverRegistry implementation that replaces the default one.
     *
     * If passing this in you don't need to pass in factRetrievers also.
     */
    factRetrieverRegistry?: FactRetrieverRegistry$1;
    /**
     * Optional persistenceContext implementation that replaces the default one.
     * This can be used to replace underlying database with a more suitable implementation if needed
     */
    persistenceContext?: PersistenceContext$1;
    logger: LoggerService;
    config: Config;
    discovery: DiscoveryService;
    database: DatabaseService;
    scheduler: SchedulerService;
    auth: AuthService;
    urlReader: UrlReaderService;
}
/**
 * @public
 * @typeParam CheckType - Type of the check for the fact checker this builder returns
 * @typeParam CheckResultType - Type of the check result for the fact checker this builder returns
 *
 * A container for exported implementations related to TechInsights.
 * FactChecker is present if an optional FactCheckerFactory is included in the build stage.
 */
type TechInsightsContext<CheckType extends Check, CheckResultType extends CheckResult> = {
    factChecker?: FactChecker<CheckType, CheckResultType>;
    persistenceContext: PersistenceContext$1;
    factRetrieverEngine: FactRetrieverEngine;
};
/**
 * @public
 *
 * Constructs needed persistence context, fact retriever engine
 * and optionally fact checker implementations to be used in the tech insights module.
 *
 * @param options - Needed options to construct TechInsightsContext
 * @returns TechInsightsContext with persistence implementations and optionally an implementation of a FactChecker
 */
declare const buildTechInsightsContext: <CheckType extends Check, CheckResultType extends CheckResult>(options: TechInsightsOptions<CheckType, CheckResultType>) => Promise<TechInsightsContext<CheckType, CheckResultType>>;

export { FactRetrieverEngine, type FactRetrieverRegistrationOptions, type FactRetrieverRegistry, type PersistenceContext, type PersistenceContextOptions, type RouterOptions, type TechInsightsContext, type TechInsightsOptions, buildTechInsightsContext, createFactRetrieverRegistration, createRouter, techInsightsPlugin as default, entityMetadataFactRetriever, entityOwnershipFactRetriever, initializePersistenceContext, techdocsFactRetriever };
